/*
 * x86 PAT support.
 * Copyright (C) 2003 Jeff Hartmann.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * JEFF HARTMANN, DAVE JONES, OR ANY OTHER CONTRIBUTORS BE LIABLE FOR ANY
 * CLAIM,  DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
 * OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
 * OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Very losely based on code I wrote at VA Linux Systems in 2001.
 *    
 * That code had the following copyright notice:
 * Copyright (C) 2001 Jeff Hartmann
 * Copyright (C) VA Linux Systems, Inc., Fremont, California.
 */

#ifndef _LINUX_PAT_H
#define _LINUX_PAT_H

#define PAT_UNCACHEABLE        0
#define PAT_WRITE_COMB         1
#define PAT_WRITE_THRGH        4
#define PAT_WRITE_PROT         5
#define PAT_WRITE_BACK         6
#define PAT_UNCACHED           7

/* Here is the PAT's default layout on ia32 cpus at boot/reset.
 * This is table 10-12 in the IA-32 Intel Architecture Software Developer's
 * Manual Volume 3: System Programming Guide.
 * PAT0: Write Back
 * PAT1: Write Through 
 * PAT2: Uncached (/ Uncacheable on Athlon cpus)
 * PAT3: Uncacheable
 * PAT4: Write Back
 * PAT5: Write Through
 * PAT6: Uncached (/ Uncacheable on Athlon cpus)
 * PAT7: Uncacheable
 *
 * Rather than mucking with the PAT entries too much, we'll only replace one
 * entry with Write Combining. Ideally, we'd do that to one of the latter 4
 * entries, but due to errata in the P3/P4 cpus, we can only use the first
 * four entries reliably.
 *
 * For that reason, we'll replace PAT1: Write Through with Write Combine.
 *
 * For more details about these errata, see Intel Pentium III Processor
 * Specification Update, errata E.27 (Upper Four PAT Entries Not Usable With
 * Mode B or Mode C Paging) and Intel Pentium IV Processor Specification 
 * Update, errata N46 (PAT Index MSB May Be Calculated Incorrectly)
 */

#define IA32_CR_PAT            0x277

/** XXX normally goes into pgtable.h **/

/* Added define for write combining page, only valid if pat enabled. */
#define _PAGE_WRTCOMB _PAGE_PWT

#define __PAGE_KERNEL_WRTCOMB \
       (_PAGE_PRESENT | _PAGE_RW | _PAGE_DIRTY | _PAGE_WRTCOMB | _PAGE_ACCESSED)

#define PAGE_KERNEL_WRTCOMB MAKE_GLOBAL(__PAGE_KERNEL_WRTCOMB)

#endif /*  _LINUX_PAT_H  */
 

